@@ -19,11 +19,12 @@ use fenris_paradis::coloring::sequential_greedy_coloring;
 use fenris_paradis::DisjointSubsets;
 use nalgebra_sparse::{pattern::SparsityPattern, CsrMatrix};
 
-use crate::allocators::{BiDimAllocator, SmallDimAllocator};
+use crate::allocators::{BiDimAllocator, DimAllocator};
 use crate::assembly::local::{
     ElementConnectivityAssembler, ElementMatrixAssembler, ElementScalarAssembler, ElementVectorAssembler,
     QuadratureTable,
 };
+use crate::nalgebra::MatrixSlice;
 use crate::space::{FiniteElementConnectivity, FiniteElementSpace};
 use crate::SmallDim;
 use fenris_sparse::ParallelCsrRowCollection;
@@ -757,7 +758,7 @@ fn add_local_to_global_<'a, T: RealField>(
 ///
 /// TODO: Move to local assembly???
 #[derive(Debug)]
-pub struct QuadratureBuffer<T, D, Data>
+pub struct QuadratureBuffer<T, D, Data = ()>
 where
     T: Scalar,
     D: DimName,
@@ -788,7 +789,7 @@ where
     T: RealField,
     GeometryDim: SmallDim,
     Data: Default + Clone,
-    DefaultAllocator: SmallDimAllocator<T, GeometryDim>,
+    DefaultAllocator: DimAllocator<T, GeometryDim>,
 {
     /// Resizes the internal buffer storages to the given size.
     pub fn resize(&mut self, quadrature_size: usize) {
@@ -813,6 +814,16 @@ where
         );
     }
 
+    pub fn populate_element_weights_and_points_from_table(
+        &mut self,
+        element_index: usize,
+        table: &(impl ?Sized + QuadratureTable<T, GeometryDim>),
+    ) {
+        let quadrature_size = table.element_quadrature_size(element_index);
+        self.resize(quadrature_size);
+        table.populate_element_quadrature(element_index, &mut self.quad_points, &mut self.quad_weights);
+    }
+
     pub fn weights(&self) -> &[T] {
         &self.quad_weights
     }
@@ -825,6 +836,10 @@ where
         &self.quad_data
     }
 
+    pub fn weights_and_points(&self) -> (&[T], &[OPoint<T, GeometryDim>]) {
+        (self.weights(), self.points())
+    }
+
     /// Calls a closure for each quadrature point currently in the workspace.
     pub fn for_each_quadrature_point<F>(&self, mut f: F) -> eyre::Result<()>
     where
@@ -915,6 +930,10 @@ impl<T: RealField> BasisFunctionBuffer<T> {
         &mut self.element_basis_values
     }
 
+    pub fn element_gradients<D: DimName>(&self) -> MatrixSlice<T, D, Dynamic> {
+        MatrixSlice::from(&self.element_basis_gradients)
+    }
+
     pub fn element_gradients_mut<D: DimName>(&mut self) -> MatrixSliceMut<T, D, Dynamic> {
         MatrixSliceMut::from(&mut self.element_basis_gradients)
     }
