@@ -1,10 +1,11 @@
-use crate::allocators::{BiDimAllocator, SmallDimAllocator, TriDimAllocator};
+use crate::allocators::{BiDimAllocator, DimAllocator, TriDimAllocator};
 use crate::assembly::global::{gather_global_to_local, BasisFunctionBuffer, QuadratureBuffer};
 use crate::assembly::local::{
     ElementConnectivityAssembler, ElementMatrixAssembler, ElementScalarAssembler, ElementVectorAssembler,
     QuadratureTable,
 };
 use crate::assembly::operators::{EllipticContraction, EllipticEnergy, EllipticOperator, Operator};
+use crate::define_thread_local_workspace;
 use crate::element::VolumetricFiniteElement;
 use crate::nalgebra::allocator::Allocator;
 use crate::nalgebra::{
@@ -13,11 +14,10 @@ use crate::nalgebra::{
 };
 use crate::space::{ElementInSpace, VolumetricFiniteElementSpace};
 use crate::util::{clone_upper_to_lower, reshape_to_slice};
-use crate::workspace::{with_thread_local_workspace, Workspace};
+use crate::workspace::with_thread_local_workspace;
 use crate::Symmetry;
 use eyre::eyre;
 use itertools::izip;
-use std::cell::RefCell;
 
 // TODO: Move this to the right spot and don't make it pub(crate)
 #[allow(non_snake_case)]
@@ -30,7 +30,7 @@ where
     T: RealField,
     SolutionDim: DimName,
     GeometryDim: DimName,
-    DefaultAllocator: BiDimAllocator<T, GeometryDim, SolutionDim>,
+    DefaultAllocator: BiDimAllocator<T, SolutionDim, GeometryDim>,
 {
     let phi_grad_ref = phi_grad_ref.into();
     let u = u.into();
@@ -162,7 +162,7 @@ where
     Space: VolumetricFiniteElementSpace<T>,
     Op: Operator<T, Space::GeometryDim>,
     QTable: ?Sized,
-    DefaultAllocator: SmallDimAllocator<T, Space::GeometryDim>,
+    DefaultAllocator: DimAllocator<T, Space::GeometryDim>,
 {
     fn solution_dim(&self) -> usize {
         Op::SolutionDim::dim()
@@ -212,7 +212,7 @@ where
     }
 }
 
-thread_local! { static WORKSPACE: RefCell<Workspace> = RefCell::new(Workspace::default());  }
+define_thread_local_workspace!(WORKSPACE);
 
 impl<'a, T, Space, Op, QTable> ElementScalarAssembler<T> for ElementEllipticAssembler<'a, T, Space, Op, QTable>
 where
@@ -220,7 +220,7 @@ where
     Space: VolumetricFiniteElementSpace<T>,
     Op: EllipticEnergy<T, Space::ReferenceDim>,
     QTable: QuadratureTable<T, Space::ReferenceDim, Data = Op::Parameters> + ?Sized,
-    DefaultAllocator: TriDimAllocator<T, Space::GeometryDim, Space::ReferenceDim, Op::SolutionDim>,
+    DefaultAllocator: TriDimAllocator<T, Op::SolutionDim, Space::GeometryDim, Space::ReferenceDim>,
 {
     fn assemble_element_scalar(&self, element_index: usize) -> eyre::Result<T> {
         let s = self.solution_dim();
@@ -259,7 +259,7 @@ where
     Space: VolumetricFiniteElementSpace<T>,
     Op: EllipticOperator<T, Space::ReferenceDim>,
     QTable: QuadratureTable<T, Space::ReferenceDim, Data = Op::Parameters> + ?Sized,
-    DefaultAllocator: TriDimAllocator<T, Space::GeometryDim, Space::ReferenceDim, Op::SolutionDim>,
+    DefaultAllocator: TriDimAllocator<T, Op::SolutionDim, Space::GeometryDim, Space::ReferenceDim>,
 {
     #[allow(non_snake_case)]
     fn assemble_element_vector_into(&self, element_index: usize, output: DVectorSliceMut<T>) -> eyre::Result<()> {
@@ -301,7 +301,7 @@ where
     Space: VolumetricFiniteElementSpace<T>,
     Op: EllipticContraction<T, Space::ReferenceDim>,
     QTable: QuadratureTable<T, Space::ReferenceDim, Data = Op::Parameters> + ?Sized,
-    DefaultAllocator: TriDimAllocator<T, Space::GeometryDim, Space::ReferenceDim, Op::SolutionDim>,
+    DefaultAllocator: TriDimAllocator<T, Op::SolutionDim, Space::GeometryDim, Space::ReferenceDim>,
 {
     #[allow(non_snake_case)]
     fn assemble_element_matrix_into(&self, element_index: usize, output: DMatrixSliceMut<T>) -> eyre::Result<()> {
@@ -372,7 +372,7 @@ where
     // We only support volumetric elements atm
     Element: VolumetricFiniteElement<T>,
     Contraction: EllipticContraction<T, Element::GeometryDim>,
-    DefaultAllocator: BiDimAllocator<T, Element::GeometryDim, Contraction::SolutionDim>,
+    DefaultAllocator: BiDimAllocator<T, Contraction::SolutionDim, Element::GeometryDim>,
 {
     assert_eq!(quadrature_weights.len(), quadrature_points.len());
     assert_eq!(quadrature_points.len(), quadrature_data.len());
@@ -468,7 +468,7 @@ where
     // We only support volumetric elements atm
     Element: VolumetricFiniteElement<T>,
     Operator: EllipticOperator<T, Element::GeometryDim>,
-    DefaultAllocator: BiDimAllocator<T, Element::GeometryDim, Operator::SolutionDim>,
+    DefaultAllocator: BiDimAllocator<T, Operator::SolutionDim, Element::GeometryDim>,
 {
     assert_eq!(quadrature_weights.len(), quadrature_points.len());
     assert_eq!(quadrature_points.len(), quadrature_data.len());
@@ -562,7 +562,7 @@ where
     // We only support volumetric elements atm
     Element: VolumetricFiniteElement<T>,
     Operator: EllipticEnergy<T, Element::GeometryDim>,
-    DefaultAllocator: BiDimAllocator<T, Element::GeometryDim, Operator::SolutionDim>,
+    DefaultAllocator: BiDimAllocator<T, Operator::SolutionDim, Element::GeometryDim>,
 {
     let s = Operator::SolutionDim::dim();
     let n = element.num_nodes();
